/**
  * rules v0.1.5
  *
  * @link https://vue-formily.netlify.app/guide/validation-rules
  * @source https://github.com/vue-formily/rules
  * (c) 2022 An Ha
  * @license MIT
  */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.Rules = {}));
})(this, (function (exports) { 'use strict';

  const _toString = Object.prototype.toString;
  function isNullOrUndefined(value) {
    return value === null || value === undefined;
  }
  /**
   * Strict object type check. Only returns true
   * for plain JavaScript objects.
   */

  function isPlainObject(obj) {
    return _toString.call(obj) === '[object Object]';
  }
  function isString(value) {
    return typeof value === 'string';
  }
  /**
   * Check if the input string contains only numeric characters.
   */

  function isNumeric(value) {
    const val = String(value);
    return val.length > 0 && !isNaN(+val);
  }
  function isFunction(fn) {
    return typeof fn === 'function';
  }
  /**
   * Returns true if the object is empty. Empty is defined as:
   * - null
   * - undefined
   * - a string with zero length
   * - an array with no elements
   * - a collection with no elements
   */

  function isEmpty(value) {
    if (isNullOrUndefined(value)) {
      return true;
    }

    let length;

    if (Array.isArray(value) || isString(value)) {
      length = value.length;
    }

    if (isPlainObject(value)) {
      length = Object.keys(value).length;
    }

    return length === 0;
  }

  // defineSchema is a utility that is primarily used for type inference
  // when declaring form schemas. Type inference is provided in the component
  // options (provided as the argument). The returned value has artificial types
  // for TSX / manual render function / IDE support.
  function defineSchema(schema) {
    return schema;
  }

  function getLength(value) {
      if (Array.isArray(value)) {
          return value.length;
      }
      if (value !== null && isPlainObject(value)) {
          return Object.keys(value).length;
      }
      if (isString(value)) {
          return value.length;
      }
      return 0;
  }

  const validator$7 = (value, { minLength = -Infinity } = {}) => {
      return getLength(value) >= minLength;
  };
  const schema$6 = defineSchema({
      validator: validator$7,
      name: 'minLength',
      cascade: true,
      for: ['string', 'enum', 'set']
  });

  const validator$6 = (value, { maxLength = Infinity } = {}) => {
      return getLength(value) <= maxLength;
  };
  const schema$5 = defineSchema({
      validator: validator$6,
      name: 'maxLength',
      cascade: true,
      for: ['string', 'enum', 'set']
  });

  const validator$5 = (value, { min = -Infinity } = {}) => {
      return value !== null && +value >= +min;
  };
  const schema$4 = defineSchema({
      validator: validator$5,
      name: 'min',
      for: ['number', 'date']
  });

  const validator$4 = (value, { max = Infinity } = {}) => {
      return value !== null && +value <= +max;
  };
  const schema$3 = defineSchema({
      validator: validator$4,
      name: 'max',
      for: ['number', 'date']
  });

  const validator$3 = (value) => !isEmpty(value);
  const schema$2 = defineSchema({
      validator: validator$3,
      name: 'required'
  });

  const validator$2 = (value) => isNumeric(value);
  const schema$1 = defineSchema({
      validator: validator$2,
      name: 'numeric',
      for: ['string']
  });

  const remail = /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
  const validator$1 = (value) => {
      return value !== null && (!value.length || remail.test(value));
  };
  const schema = defineSchema({
      validator: validator$1,
      name: 'email',
      for: ['string']
  });

  const validator = (value) => !isEmpty(value);
  var requiredIf = (condition) => {
      return defineSchema({
          validator(value, ...args) {
              const shouldValidate = isFunction(condition) ? condition.call(this, value, ...args) : condition;
              return shouldValidate ? validator.call(this, value) : true;
          },
          name: 'requiredif'
      });
  };

  exports.email = schema;
  exports.max = schema$3;
  exports.maxLength = schema$5;
  exports.min = schema$4;
  exports.minLength = schema$6;
  exports.numeric = schema$1;
  exports.required = schema$2;
  exports.requiredIf = requiredIf;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
